
; Returns vector with mag data or -1 on error
;
function thm_part_slice2d_getmag, ds, mag_data=mag_data, $
                                  trange=trange, $
                                  rotation=rotation, $
                                  fail=fail

    compile_opt idl2, hidden
  
  
  
  if keyword_set(mag_data) then begin
  
    ; user specified tplot variable 
    bfield = dat_avg(mag_data, trange[0], trange[1], /interp)
  
    if total(finite(bfield,/nan)) gt 0 then begin
      fail = 'Invalid magnetic field variable: "'+mag_data+'".'
      dprint, dlevel=1, fail
      return, -1
    endif
  
  endif else begin
    
    ; data added in thm_part_dist_array
    for i=0, n_elements(ds)-1 do begin
      ; only use data in requested time range
      times_ind = thm_part_slice2d_intrange(ds[i], trange, n=ndat)
      if ndat gt 0 then begin
        bdata = ~keyword_set(bdata) ? [ (*ds[i])[times_ind].magf ]   :    $
                                      [ [bdata], [(*ds[i])[times_ind].magf] ]
      endif
    endfor 
    
    if ~keyword_set(bdata) then begin
      fail = 'Unknown error retrieving B field data. '+ $
             'Please report this to the TDAS development team.' 
      dprint, dlevel=1, fail
      return, -1
    endif
    
    ;average combined data
    bfield = average(bdata, 2)
    
  endelse
  
  return, bfield

end



;Returns bulk velocity vector or -1 on error
;
;Note: thm_esa_slice2d Divides total flux (over entire time range) by 
;   the total density; here the flux/density was previously 
;   calculated for each distribution and is now averaged.
function thm_part_slice2d_getvel, ds, vel_data=vel_data, $
                                trange=trange, $
                                rotation=rotation, $
                                fail=fail

    compile_opt idl2, hidden
  
  
  
  
  if keyword_set(vel_data) then begin
  
    ; user specified variable
    vbulk = dat_avg(vel_data, trange[0], trange[1], /interp)

    if total(finite(vbulk,/nan)) gt 0 then begin
      fail = 'Invalid velocity variable: "'+vel_data+'".'
      dprint, dlevel=1, fail
      return, -1
    endif

  endif else begin
  
  
    ; data added in thm_part_dist_array
    for i=0, n_elements(ds)-1 do begin 
      ; only use data in requested time range
      times_ind = thm_part_slice2d_intrange(ds[i], trange, n=ndat)
      if ndat gt 0 then begin
        vdata = ~keyword_set(vdata) ? [ (*ds[i])[times_ind].velocity ]  :  $
                                      [ [vdata], [(*ds[i])[times_ind].velocity] ]
      endif
    endfor 
    
    if ~keyword_set(vdata) then begin
      fail = 'Unknown error retrieving bulk velocity data. '+ $
             'Please report this to the TDAS development team.' 
      dprint, dlevel=1, fail
      return, -1
    endif
  
    ; average combined data
    vbulk = average(vdata, 2) ;vel in km/s
  
  endelse

  ; convert to m/s
  return, vbulk * 1000.

end



;Returns sun vector or 0 on error
;
; If multiple distributions are being used the sun vector 
; will be determined by averaging.  In practice, it should 
; not change appreciably from one distribution to the next.
; This is done largely to be consistent with the treatment of  
; the bulk velocity and B-field vectors, but it also provides
; a place to error check.
; 
function thm_part_slice2d_getsun, ds, trange=trange, fail=fail

    compile_opt idl2, hidden


  ; get sun vector data added in thm_part_dist_array
  for i=0, n_elements(ds)-1 do begin
    
    if ~in_set( 'sun_vector', strlowcase(tag_names(*ds[i])) ) then continue 
    
    ; only use data in requested time range
    times_ind = thm_part_slice2d_intrange(ds[i], trange, n=ndat)
    if ndat gt 0 then begin
      svs = ~keyword_set(svs) ? [ (*ds[i])[times_ind].sun_vector ]  :  $
                                [ [svs], [(*ds[i])[times_ind].sun_vector] ]
    endif
  endfor 
  
  ; assume sun vector was not requested if no data is present
  if ~keyword_set(svs) then begin
    return, 0
  endif

  ; average
  sun_vector = average(svs, 2) 
  
  
;  ;TESTING ******************  
;  ;Check that angle between vectors is not too large
;  rsvs = reform(sun_vector) # replicate(1.,dimen2(svs))
;  ca = total(  svs * rsvs ,1)     /      $
;       ( sqrt(total(svs^2,1)) * sqrt(total(rsvs^2,1)) ) 
;  a = acos(ca < 1) * 180/!pi * 60
;  print, string(10b) + 'sun vec diffs from ave (minutes)' 
;  print, a
;  ;**************************
  
  
  ; Check for NaNs that could signify an error
  if in_set(finite(sun_vector), 0) then begin
    dprint, dlevel=0, 'Error: Invalid sun vector retrieved for '+ $
                      time_string(trange[0])+' to '+time_string(trange[1]) 
    return, 0
  endif


  return, sun_vector

end




;Performs rotations from DSL to requested coordinates (COORD keyword)
; *Assumes the transformation from the data's native DSL to
;  the requested coords (GSM, GSE) is invariant over the
;  time range of the slice.* 
pro thm_part_slice2d_cotrans, vectors, bfield=bfield, vbulk=vbulk, sunvec=sunvec, $
                              type=type, probe=probe, coord=coord, twin=twin, $
                              matrix=matrix, $
                              fail=fail

    compile_opt idl2, hidden


  ctn = '2dslice_temp_cotrans'
  if stregex(coord, '^dsl$', /bool, /fold_case) then return
  if ~keyword_set(probe) then begin
    fail = 'Cannot determine spacecraft. Coordinate transform canceled.'
    dprint,dlevel=1, fail
    return
  endif
  if ~stregex(coord, '(^gsm$)|(^gse$)', /bool, /fold_case) then begin
    fail = 'Invalid coordinate keyword: '+coord
    dprint,dlevel=1, fail
    return
  endif
  probe = strlowcase(probe)
  coord = strlowcase(coord)

  dprint, dlevel=4, 'Rotating to '+strupcase(coord)+' coordinates'

  ;Load necessary support data (2 min padding at each end)
  thm_load_state, probe=probe, trange = twin + 120*[-1,1], suffix='_'+ctn, $
                  /get_support_data, /keep_spin_data, verbose=0


  ;Transform x,y,z unit vectors to new coordinates
  store_data, ctn, data = {x:replicate(mean(twin),3), $
                           y: [ [1.,0,0], [0,1,0], [0,0,1] ] }, verbose=0
  thm_cotrans, ctn, probe=probe, in_coord='dsl', out_coord=coord, $
               support_suffix='_'+ctn, out_suff='_'+coord


  ;Form transformation matrix from transformed basis
  get_data, ctn+'_'+coord, data=ctd
  if size(ctd,/type) ne 8 then begin
    fail = 'Could not obtain coordinate transform from DSL -> '+strupcase(coord)+ $
           ', STATE data may be absent for the requested period.'
    dprint, dlevel=1, fail
    store_data, '*'+ctn+'*', /delete, verbose=0 ;delete temp vars
    return
  endif
  matrix = transpose(ctd.y)


  ;Apply to velocity data and required rotation vectors
  if keyword_set(type) then vectors = matrix ## temporary(vectors)
  if keyword_set(vbulk) then vbulk = matrix ## vbulk
  if keyword_set(bfield) then bfield = matrix ## bfield
  if keyword_set(sunvec) then sunvec = matrix ## sunvec


  ;Delete temporary tplot data
  store_data, '*'+ctn+'*', /delete, verbose=0

end



;Retrieves rotation matrix for FAC
; *Assumes the transformation is invariant over the slice's time range* 
pro thm_part_slice2d_fac, rotation, probe=probe, mag_data=mag_data, $
                          twin=twin, matrix=matrix, fail=fail

    compile_opt idl2, hidden


  ctn = '2dslice_temp_fac'
  if ~keyword_set(probe) then begin
    fail = 'Cannot determine spacecraft. FAC transform canceled.'
    dprint,dlevel=1, fail
    return
  endif
  if ~keyword_set(mag_data) || ~keyword_set(tnames(mag_data)) then begin
    fail = 'Cannot perform FAC rotation without B-field data. '+ $
           'Use MAG_DATA keyword to pass in valid tplot variable.'
    dprint, dlevel=1, fail
    return 
  endif

  probe = strlowcase(probe)
  rotation = strlowcase(rotation)

  dprint, dlevel=4, 'Rotating to FAC using '+rotation

  ;Get averaged bfield data
  get_data, mag_data, dlimits = mdl ;will need dlimits later
  bfield = dat_avg(mag_data, twin[0], twin[1], /interp)
  if total(finite(bfield,/nan)) gt 0 then begin
    fail = 'Cannot perform FAC rotation.  The data in '+ $
           mag_data+' is out of range.'
    dprint, dlevel=1, fail
    return
  endif
  
  ;Store averaged bfield data for thm_fac_matrix_make
  store_data, 'th'+probe+ctn+'_bfield', verbose=0, dlimits=mdl, $
               data = {x:mean(twin), y: reform(bfield,1,n_elements(bfield))} 

  ;Load necessary support data (5 min padding at each end)
  thm_load_state, probe=probe, trange = twin + 300*[-1,1], $ ;, suffix='_'+ctn, $
                  /get_support_data, /keep_spin_data, verbose=0


  ;Create rotation matrix
  thm_fac_matrix_make, 'th'+probe+ctn+'_bfield', other_dim=rotation, $
                       pos_var_name='th'+probe+'_state_pos', $ ;_'+ctn, $
                       newname = ctn+'_mat'

  ;Get rotation matrix from tplot variable
  get_data, ctn+'_mat', data=ctd
  if size(ctd,/type) ne 8 then begin
    fail = 'Could not obtain FAC ('+rotation+') transform'+ $
           ', STATE data may be absent for the requested period.'
    dprint, dlevel=1, fail
    store_data, '*'+ctn+'*', /delete, verbose=0 ;delete temp vars
    return
  endif
  matrix = transpose(ctd.y)

  ;Delete temporary tplot data
  store_data, '*'+ctn+'*', /delete, verbose=0

end



; Performs rotation from the standard coordinates (set with COORD keyword)
; to the user specified coordinates (set with ROTATION keyword) 
pro thm_part_slice2d_rotate, velocities, bfield=bfield, vbulk=vbulk, sunvec=sunvec, $
                             type=type, rotation=rotation, mag_data=mag_data, $
                             twin=twin, probe=probe, matrix=matrix, $
                             fail=fail

    compile_opt idl2, hidden


  ; Create/get rotation matrix
  case strlowcase(rotation) of
    'bv': matrix=cal_rot(bfield,vbulk)
    'be': matrix=cal_rot(bfield,crossp(bfield,vbulk))
    'xy': matrix=cal_rot([1,0,0],[0,1,0])
    'xz': matrix=cal_rot([1,0,0],[0,0,1])
    'yz': matrix=cal_rot([0,1,0],[0,0,1])
    'xvel': matrix=cal_rot([1,0,0],vbulk)
    'perp': matrix=cal_rot(crossp(crossp(bfield,vbulk),bfield),crossp(bfield,vbulk))
    'perp_yz': matrix=cal_rot(crossp(crossp(bfield,[0,1,0]),bfield),crossp(crossp(bfield,[0,0,1]),bfield))
    'perp_xy': matrix=cal_rot(crossp(crossp(bfield,[1,0,0]),bfield),crossp(crossp(bfield,[0,1,0]),bfield))
    'perp_xz': matrix=cal_rot(crossp(crossp(bfield,[1,0,0]),bfield),crossp(crossp(bfield,[0,0,1]),bfield))
    else: begin
    
      ;handle FAC rotations through thm_fac_matrix_make
      valid_fac_coords = ['xgse','ygsm','zdsl', 'rgeo','mrgeo', 'phigeo','mphigeo', 'phism','mphism']
      if total(stregex(valid_fac_coords, rotation, /bool, /fold_case)) gt 0 then begin    
        thm_part_slice2d_fac, rotation, probe=probe, mag_data=mag_data, $
                              twin=twin, matrix=matrix, fail=fail                        
        if keyword_set(fail) then return
      endif else begin
        fail = 'Unrecognized rotation: "'+rotation+'".'
        dprint, dlevel=1, fail
        return
      endelse
      
    end
  endcase
  
  ; Check that matrix was formed correctly
  if total( finite(matrix,/nan) ) gt 0 then begin
    fail = 'Cannot form rotation matrix, B-field and/or Bulk Velocity variables may contain NaNs.'
    dprint, dlevel=1, fail
    return
  endif
  
  ; Prevent data from being mutated to doubles
  matrix = float(matrix)

  if rotation ne 'xy' then begin
    dprint, dlevel=4, 'Alligning slice plane to ' +rotation
  endif else return
  
  ; Rotate data
  if keyword_set(type) then velocities = matrix ## temporary(velocities)
  if keyword_set(vbulk) then vbulk = matrix ## vbulk
  if keyword_set(bfield) then bfield = matrix ## bfield
  if keyword_set(sunvec) then sunvec = matrix ## sunvec

end


; Perform rotation from user specified coordinates to the 
; slice plane's coordinates (determined by SLICE_NORM and SLICE_X)
pro thm_part_slice2d_orientslice, velocities, vbulk=vbulk, sunvec=sunvec, $
                                  type=type, slice_x=slice_x, slice_z=slice_z, $
                                  matrix = matrix, fail=fail
                                  
    compile_opt idl2, hidden


  tolerance = 5d-5 ;arbitrary


  ;check z-axis input
  if keyword_set(slice_z) then begin
    if n_elements(slice_z) ne 3 then begin
      fail='Invalid slice normal, please specify a valid 3-vector'
      dprint, dlevel=1, fail
      return
    endif
  endif else begin
    slice_z = [0,0,1d]
  endelse


  ;copy slice normal and normalize
  z = double(slice_z)
  z = z/norm(z)


  ;check x-axis input
  if keyword_set(slice_x) then begin
    if n_elements(slice_x) ne 3 then begin
      fail='Invalid slice x-axis, please specify a valid 3-vector'
      dprint, dlevel=1, fail  
      return
    endif
  ;if x-axis not specified use projection of ordinate closest to the slice plane
  endif else begin
    
    ;get angle between norm and each ordinate
    ord = [ [1,0,0], [0,1,0], [0,0,1]  ]
    angles = (180/!dpi) * acos( [ total(ord[*,0]*z), $
                                  total(ord[*,1]*z), $
                                  total(ord[*,2]*z)  ] )
    
    ;use ordinate that is closest to perpendicular with norm
    dummy = min( abs(angles - 90), ind)
    
    slice_x = ord[*,ind]
  
    dprint, dlevel=4, 'Slice x-axis not set, using projection of default '+ $
                      (['x','y','z'])[ind] + '-axis)'
  endelse
  

  ;normalize vector used to define slice's x-axis
  xp = double(slice_x)
  xp = xp/norm(xp)
  
  
  ;get slice's y axis
  y = crossp(z,xp)
  
  if norm(y) lt tolerance then begin
    fail='Cannot orient slice plane, x-axis and slice '+ $
         'normal are too close to being parallel.'
    dprint, dlevel=1, fail
    return
  endif

  y = y/norm(y)
  
  
  ;get slice's x axis
  x = crossp(y,z)
  
  if norm(x) lt tolerance then begin ;this should never be true
    fail='Cannot orient slice plane due to an unkown error.'
    dprint, dlevel=0, fail
    return
  endif
  
  x = x/norm(x)


  ;get rotation matrix
  ;use single precision to keep input data from being mutated to double
  matrix = float([[x],[y],[z]])
  
  
  ;rotate data
  if keyword_set(type) then velocities = matrix ## temporary(velocities)
  if keyword_set(vbulk) then vbulk = matrix ## vbulk
  if keyword_set(sunvec) then sunvec = matrix ## sunvec

end


; Subtracts bulk velocity vector from velocities array
; todo: fix for geo
pro thm_part_slice2d_subtract, velocities, vbulk, vel_data=vel_data

    compile_opt idl2, hidden

  if keyword_set(vel_data) then begin
    dprint, dlevel=4,'Velocity used for subtraction is '+vel_data
  endif else begin
    dprint, dlevel=4, 'Velocity used for subtraction is V_3D'
  endelse

  velocities[*,0] = velocities[*,0] - vbulk[0]
  velocities[*,1] = velocities[*,1] - vbulk[1]
  velocities[*,2] = velocities[*,2] - vbulk[2]
  

end



; Smooths slice output using a 2D gaussian convolution.
; Smoothing window should be an odd # of points >= 3
; 
pro thm_part_slice2d_smooth, part_slice, width

    compile_opt idl2, hidden


  smooth = round(width)

  if smooth ge 2 then begin
    
    ;ensure kernel with odd # of elements >= 3
    n = smooth > 3.
    if n mod 2 eq 0 then n--
    
    ;Increase the STDV with the smoothing width to allow CONVOL to
    ;effectively smooth over a greater area. Simply extending the
    ;width with const. STDV adds negligable terms to the kernel.
    ;Allowing the STDV to grow too quickly results in a flat surface.
    c = alog(n) - .25  ;Linear functions were tested but still resulted
                       ;in square artifacts. This should allow the 
                       ;width of the gaussian to increase while 
                       ;also adding smallers terms to the edges.
    
    ;set up for kernel
    s = findgen(n) - floor(n/2.)
    sx = s ## replicate(1.,n)
    sy = s # replicate(1.,n)
    
    ;2D normalized gaussian kernel
    kernel = exp( (-.5) * ((sx/c)^2 + (sy/c)^2) )  ;/  (2*!pi*c^2)
    kernel = kernel / total(kernel)
    
    part_slice = convol(part_slice, kernel, /edge_zero)
    
;    part_slice = smooth(part_slice, smooth)
;    negidx = where(part_slice lt 0, nneg)
;    if nneg gt 0 then begin
;      part_slice[negidx] = 0
;    endif

  endif else begin
    dprint, dlevel=1, 'Smoothing not applied.  Smoothing value must be >= 2'
  endelse

  return

end



; Return indeces to structures that are within the 
; requested time range.  This is done a number of 
; times and so the code has been moved here.
function thm_part_slice2d_intrange, ds, trange, n=n

    compile_opt idl2, hidden

  times = (*ds).time + ((*ds).end_time - (*ds).time)/2  ;use center
  times_ind = where(times ge trange[0] AND times le trange[1], n)

  return, times_ind
  
end



;+
;Procedure: thm_part_slice2d
;
;Purpose: Returns a 2-D slice of the 3-D THEMIS ESA/SST ion or electron distribution
;         function.  The 2-D slice is returned via the PART_SLICE, XGRID, YGRID, and 
;         SLICE_INFO keywords.
;         
;         This procedure works in conjunction with thm_part_dist_array.pro and 
;         plot_part_slice2d.pro.  The corresponding crib is thm_crib_part_slice2d.pro 
;
;   There are three methods for generating slices for plotting:
;
;   Geomtric:
;     Exact method showing the bounds of all bins intersecting the slice plane.
;     Values are averaged in cases of overlapping bins or when the slice plane 
;     is coplanar with a bin's boundary.  
;
;   2D Interpolation (from thm_esa_slice2d):
;     Datapoints within the specified theta or z-axis range are projected onto 
;     the slice plane and linearly interpolated onto a regular 2d grid. 
;     X,Y-range values determined slightly differently for 2D Interpolation
;
;   3D Interpolation:
;     The entire 3-dimensional distribution is linearly interpolated onto a 
;     regular 3d grid and a this slice is extracted.  The slice will consist of
;     up to resolution^2 datapoints from within 0.866/resolution of the total 
;     velocity range along the normal. This method works best with dense data.
;       Note: - Not all points within the aforemention range will be used.
;             - Interpolation may occur across data gaps or areas with recorded zeroes
;             - Higher resolution regridding will severely slow this method
;     
;     
;    
;Callins Sequence:
;    thm_part_slice2d, datArr, [datArr2, [datArr3, [datArr4]]], $
;                      timewin = timewin, slice_time=slice_time, $
;                      part_slice=part_slice, $
;                      fail=fail
;
;
;Arguments:
; DATARR[#]: An array of pointers to 3D data structures.
;            See thm_part_dist_array.pro for more.
; 
; 
;Input Keywords:

; SLICE_TIME: Beginning of time window in seconds since Jan. 1, 1970.  If
;             CENTER_TIME keyword set, then TIME is the center of the time widow
;             specified by the TIMEWIN keyword.
; TIMEWIN: Length in seconds over which to compute the slice.
; CENTER_TIME: Flag that, when set, centers the time window around the time 
;              specified by SLICE_TIME keyword. 
; 
; COORD: A string designating what coordinate system the slice will be based off.
;        Default is 'DSL', but can also be 'GSE' or 'GSM'
; ROTATION: The rotation keyword further specifies the coordinates system used
;           to create a slice. A description of each new set of cartesian coordinates
;           is given below.  Some rotations are invariant of the coordinates
;           specified by COORD (BV,perp), other are not (xy,perp_xy). By default
;           the slice will be cut along the x-y plane of the given coordinates.
;                      
;           Field alligned coordinates (FAC) are also available.
;           See thm_fac_matrix_make for available options.  Any valid input to
;           the OTHER_DIM keyword may be used (examples below).
;
;       Special Coordinates
;       'BV': The x axis is parallel to the B field; the bulk velocity defines the x-y plane.
;       'BE': The x axis is parallel to the B field; the B x V(bulk) vector defines the x-y plane.
;       'xy'/'xyz': (Default) The x axis is V_x and the y axis is V_y.
;       'xz': The x axis is V_x and the y axis is V_z.
;       'yz': The x axis is V_y and the y axis is V_z.
;       'xvel': The x axis is V_x; the y axis is defined by the bulk velocity. 
;       'perp': The x axis is the bulk velocity projected onto the plane normal to the B field; y is B x V(bulk)
;       'perp_xy': Geometric xy coordinates projected onto the plane normal to the B field.
;       'perp_xz': Geometric xz coordinates projected onto the plane normal to the B field.
;       'perp_yz': Geometric yz coordinates projected onto the plane normal to the B field.
;
;       Field Alligned Coordinates
;       'xgse':  The x axis is the projection of the GSE x-axis
;       'ygsm':  The y axis is the projection of the GSM y-axis
;       'zdsl':  The y axis is the projection of the DSL z-axis
;       'RGeo':  The x is the projection of radial spacecraft position vector (GEI)
;       'mRGeo':  The x axis is the projection of the negative radial spacecraft position vector (GEI)
;       'phiGeo':  The y axis is the projection of the azimuthal spacecraft position vector (GEI), positive eastward
;       'mphiGeo':  The y axis is the projection of the azimuthal spacecraft position vector (GEI), positive westward
;       'phiSM':  The y axis is the projection of the azimuthal spacecraft position vector in Solar Magnetic coords
;       'mphiSM':  The y axis is the projection of the negative azimuthal spacecraft position vector in Solar Magnetic coords
;
; 
; SLICE_X/SLICE_NORM: These keywords respectively specify the slice plane's 
;               x-axis and normal within the coordinates specified by 
;               COORD and ROTATION. Both keywords take 3-vectors as input.
;               
;               If SLICE_X is not specified then the given coordinate's 
;               x-axis will be used. If SLICE_X is not perpendicular to 
;               the normal it's projection onto the slice plane will be used.
;               An error will be thrown if no projection exists.
;               
;               If SLICE_NORM is not specified then the given coordinate's
;               z-axis will be used (slice along by x-y plane in those 
;               coordinates).
;                 
;       examples:
;         Slice plane perpendicular to DSL z-axis using [3,2,0] as plane's x-axis: 
;         (this is the same as only using SLICE_X=[3,2,1])
;           COORD='dsl' (default), ROTATION='xyz' (default), SLICE_X=[3,2,1]
;         
;         Slice plane perp. to GSE x-axis, bulk velocity used to define plane's x-axis:
;           COORD='gse', ROTATION='xvel', SLICE_NORM=[1,0,0], SLICE_X=[0,1,0]
;
;         Slice plane along the B field and radial position vectors, B field used as slice's x-axis:
;           ROTATION='rgeo', SLICE_NORM=[0,1,0], SLICE_X=[0,0,1]
;
; DISPLACEMENT: Value in m/s that specifies how far from the origin the slice 
;               plane will be cut.
;               
;       example:
;         Slice plane cut at Z_gse = 500
;           COORD='gse', ROTATION='xyz' (default), DISPLACEMENT=500.
;
;
; TWO_D_INTERP: Flag to use 2D interpolation method (described above)
; THREE_D_INTERP: Flag to use 3D interpolation method (described above)
;
;
; AVERAGE_ANGLE: Two element array specifying an angle range over which 
;                averaging will be applied. The angle is measured 
;                from the slice plane and about the slice's x-axis; 
;                positive in the right handed direction. This will
;                average over all data within that range.
;                    e.g. [-25,25] will average data within 25 degrees
;                         of the slice plane about it's x-axis
;
; UNITS: A string designating the desired units ('Counts','DF','Rate','CRate',
;        'Flux','EFlux','E2Flux','E3Flux')
;        Default = DF
; COUNT_THRESHOLD: Mask out bins that fall below this number of counts BEFORE averaging.
;                (e.g. COUNT_THRESHOLD=1 masks bins with counts below 1)
; FRACTIONAL_COUNTS: Flag to keep the ESA unit conversion routine from rounding 
;                    to an even number of counts when removing the dead time 
;                    correction (no effect if input data already in counts, 
;                    no effect on SST data).
; RESOLUTION: A single integer specfying the resolution of each dimension of the
;             slice (as well as the interpolated volume in the the 3d case).
;             Defaults:  2D Interpolation: 51
;                        3D Interpolation: 150
;                               Geometric: 500 
; ERANGE: Two element array specifying the energy range to be used
; SMOOTH: An odd integer >=3 specifing the width of the smoothing window in # 
;         of points. Even entries will be incrimented, 0 and 1 are ignored.
;         Smoothing is performed with a gaussian convolution.
; 
; 
; REGRID: (2D/3D Interpolation only)
;         A three element array specifying regrid dimensions in phi, theta, and 
;         energy respectively. If set, all distributions' data will first be 
;         spherically interpolated using the nearest neighbor method. For example  
;         the default [64,32,64] will yield distributions with 63x32 points per 
;         energy, and 64 energy bins). Increasing the regridding resolution will
;         slow slice production, particularly if using 3D interpolation.
;         Default = [64,32,64]
; THETARANGE: (2D interpolation only)
;             Angle range, in degrees [-90,90], used to calculate slice.
;             Default = [-20,20]; will override ZDIRRANGE. 
; ZDIRRANGE: (2D interpolation only)
;            Z-Axis range, in km/s, used to calculate slice.
;            Ignored if called with THETARANGE.
;
; MSG_OBJ: Object reference to GUI message bar. If included useful
;          console messages will also be output to GUI.
; 
;
;Output:
; PART_SLICE: Structure to be passed to plot_part_slice2d.
;      {
;       data: two dimentional array (NxN) containing the datato be plotted (the slice)
;       xgrid: N dimentional array of x-axis values for plotting 
;       ygrid: N dimentional array of y-axis values for plotting 
;       probe: string containing the probe
;       dist: string (or array of) containing the type of distribution used
;       coord: string describing the coordinate system used for the slice
;       rot: string describing the user specified rotation (N/A for 2D interp)
;       units: string describing the units
;       twin: string describing the time window of the slice
;       range: array containing the range of the un-interpolated data 
;       vrange: array containing the velocity range of the instrument(s)
;       trange: array containing the numerical time range
;       shift: 3-vector containing any translations made in addition to 
;              requested rotations (e.g. subtracted bulk velocity)
;       bulk: 3-vector containing the bulk velocity in the slice plane's coordinates
;       sunvec: 3-vector containing the sun direction in the slice plane's coordinates
;       coord_m: Rotation matrix from original data's coordinates (DSL) to
;                those specified by the COORD keyword.
;       rot_m: Rotation matrix from the the specified coordinates to those 
;            defined by ROTATION.
;       orient_m: Rotation matrix from the coordinates defined by ROTATION to 
;                 the coordinates defined by SLICE_NORM and SLICE_X 
;                 (column matrix of new coord's basis).
;       }
;
;
;CAVEATS: Due to IDL software constraints regions containing no data 
;         are assigned zeros instead of NaNs.  
;
;
;CREATED BY: A. Flores
;            Based on work by Bryan Kerr and Arjun Raj 
;
;EXAMPLES:  see the crib file: thm_crib_part_slice2d.pro
;-

pro thm_part_slice2d, ptrArray, ptrArray2, ptrArray3, ptrArray4, $
                    ; Time options
                      timewin=timewin, slice_time=slice_time, $
                      center_time=center_time, $
                    ; Range limits
                      erange=erange, $
                      thetarange=thetarange, zdirrange=zdirrange, $
                    ; Orientations
                      coord=coord, rotation=rotation, $
                      slice_x=slice_x, slice_norm=slice_z, $
                      displacement=displacement, $  
                    ; Support Data
                      mag_data=mag_data, vel_data=vel_data, $
                    ; Type options
                      type=type, two_d_interp=two_d_interp, $
                      three_d_interp=three_d_interp, $
                    ; Other options
                      units=units, resolution=resolution, $
                      average_angle=average_angle, smooth=smooth,  $
                      count_threshold=count_threshold, subtract=subtract, $
                      regrid=regrid, slice_width=slice_width, $
                    ; Output
                      part_slice=slice_struct, $
                    ; Dummy vars to conserve backwards compatibility 
                    ; (should allow oldscripts to be run withouth issue)
                      xgrid=xgrid, ygrid=ygrid, slice_info=slice_info, $
                      onecount=onecount, $
                    ; Other
                      msg_obj=msg_obj, fail=fail, $
                      _extra = _extra

    compile_opt idl2

if size(ptrArray,/type) ne 10 then begin
  fail = 'Invalid data structure.  Input must be valid array of pointers to arrays of '+ $
          'ESA or SST distributions with required tags. '+ $
          'See thm_part_dist_array.'
  dprint, dlevel=1, fail
  return
endif

if ~keyword_set(slice_time) then begin
  fail = 'Please specifiy a time at which to compute the slice.'
  dprint, dlevel=1, fail
  return
endif

if ~keyword_set(timewin) then begin
  fail = 'Please specifiy a time window for the slice."
  dprint, dlevel=1, fail
  return
endif

d_set = [keyword_set(ptrArray), keyword_set(ptrArray2), $
          keyword_set(ptrArray3), keyword_set(ptrArray4)]

if total(d_set) gt 1 and keyword_set(type) then begin
  fail = 'Multiple distributions are only supported for the '+ $
         'geometric method.'
  dprint, dlevel=1, fail
  return
endif



fail = ''


; Defaults
;---------
if ~keyword_set(coord) then coord='dsl'
if ~keyword_set(rotation) then rotation='xy'
if ~keyword_set(units) then units = 'df'
if ~keyword_set(slice_z) then slice_z = [0,0,1.]
if ~keyword_set(displacement) then displacement = 0.
if rotation eq 'xyz' then rotation = 'xy'
if rotation eq 'perp_xyz' then rotation = 'perp_xy'
probe = keyword_set((*ptrArray[0])[0].spacecraft) ? (*ptrArray[0])[0].spacecraft : $
                 strmid((*ptrArray[0])[0].project_name, 0, /reverse_offset)
if ~stregex(probe, '[abcde]', /bool, /fold_case) then probe = ''

;Type specific:
if keyword_set(two_d_interp) then type = 2
if keyword_set(three_d_interp) then type = 3
if size(type,/type) eq 0 then type = 0
if type gt 3 then type = 0

if keyword_set(type) && keyword_set(average_angle) then begin
  dprint, dlevel=1, 'Angular averaging is only applicable to the geometric method.'+ $
                    'No averaging will be applied.'
  return
endif

; 2d interp
if type eq 2 then begin 
  if size(smooth,/type) eq 0 then smooth = 3
  if size(resolution,/type) eq 0 then resolution = 51
  if ~keyword_set(thetarange) and ~keyword_set(zdirrange) then begin
    thetarange = [-20.,20]
  endif
endif 

; 2d nn
if type eq 1 then begin
  if size(resolution,/type) eq 0 then resolution = 250
  if keyword_set(regrid) && n_elements(regrid) eq 3 then begin
    regrid = float(regrid)
  endif else $
    regrid = [64.,32,64]
endif

; geometric
if ~keyword_set(type) then begin
  ;nothing for now
endif

;resolution
if ~keyword_set(resolution) then begin
  resolution = keyword_set(type) ? 150:500 
endif

;preserve backward compatability
if ~keyword_set(count_threshold) && keyword_set(onecount) then count_threshold = onecount



msg = 'Processing slice at ' + time_string(slice_time, format=5) +'... '
dprint, dlevel=2, msg 
if obj_valid(msg_obj) then msg_obj->update, msg 



; Account for multiple data types & mode changes.
; 
; Each ptrArray[#] input will be a pointer or pointer array.
; The user may pass in up to 4 of such variables if requesting
; multiple data types (e.g. psif & peif).  If the data spans 
; a mode change then ptrArray[#] will be an array.
; 
; All data is consolidated here and looped over in
; the next section.
; -------------------------------
temp=where(d_set,c)

if c eq 1 then begin
  ds = [ptrArray]
endif else if c eq 2 then begin
  ds = [ptrArray,ptrArray2]
endif else if c eq 3 then begin
  ds = [ptrArray,ptrArray2,ptrArray3]
endif else if c eq 4 then begin
  ds = [ptrArray,ptrArray2,ptrArray3,ptrArray4]
endif else begin
  fail = "Error: This should never happen, please report to the TDAS software team."
  dprint,dlevel=0, fail
  return
endelse


; Get time info
;--------------

; get the boundaries of the time window
if keyword_set(center_time) then begin
  trange = [slice_time - timewin/2, $
            slice_time + timewin/2  ]
endif else begin
  trange = [slice_time, $
            slice_time + timewin ]
endelse



; Produce a concatenated list of cartesian-based
; velocity bins (and associated data points) by 
; looping over structure types. Different structure 
; types may be the result of mode changes or 
; multiple data types being used.
; 
; If geometric mode is being used (type=0) then
; dphi, dtheta, and dr arrays are also returned.
;-------------------------------
for i=0, n_elements(ds)-1 do begin

  ; get indexes of dat structures in reqested time window
  times_ind = thm_part_slice2d_intrange(ds[i], trange, n=ndat)
  if ndat eq 0 then begin
    missing = keyword_set(missing) ? missing++:1  ;track # of out of range distributions
    dprint, dlevel=4, 'No '+(*ds[0])[0].data_name+' data in time range: ' + $
                      time_string(slice_time)+ ' + '+strtrim(timewin,2)+' sec.'
    continue
  endif

  ;Get cartesian velocities
  thm_part_slice2d_getXYZ, (*ds[i])[times_ind], units, type, $
                           count_threshold=count_threshold, regrid=regrid, erange=erange, $
                           orange=org, $         ;range of constrained data in specified units
                           vrange=vrange, $      ;range of constrained velocities
                           velocities=v0, $           ;array of velocities
                           datapoints=dpoints, $      ;arraw of data
                           dp=dp0, dt=dt0, dv=dv0, $  ;dphi, dtheta, dr
                           fail=fail, $
                           _extra=_extra
  if keyword_set(fail) then return

  ;Concatenate velocities from different distributions
  if keyword_set(velocities) then begin
    velocities = [ velocities, temporary(v0)]
    datapoints = [datapoints, temporary(dpoints)]
  endif else begin
    velocities = temporary(v0)
    datapoints = temporary(dpoints)
  endelse

  ;concatenate extra variables for geometric method
  if keyword_set(dp0) then begin
    if keyword_set(dp) then begin
      dp = [ dp, temporary(dp0)]
      dt = [ dt, temporary(dt0)]
      dv = [ dv, temporary(dv0)]
    endif else begin
      dp = dp0
      dt = dt0
      dv = dv0
    endelse
  endif

  orange = keyword_set(orange) ? minmax([orange,org]):org

endfor


; If there was no data within the specified time range then 
; the datapoints variable should be empty.
if ~keyword_set(datapoints) then begin
  fail = (keyword_set(missing) && missing eq n_elements(ds))  ?  $
      'No distributions exist within the given time window: '+ $
         time_string(slice_time)+ ' + '+strtrim(timewin,2)+' sec).'  :  $
      'Error extracting data from particle distributions. '+ $
         'If using 2D interpolation try extending the THETA or Z range.'
  
  dprint, dlevel=1, fail
  return
endif



; For testing only: shows 3-D distribution of nonzero data
if keyword_set(vis) then begin
  rr = where(datapoints ne 0)
  rg = [-max(velocities),max(velocities)]
  iplot, velocities[rr,0], velocities[rr,1], velocities[rr,2], linestyle=6, sym_index=1, $
         xrange = rg, yrange=rg, zrange=rg, $
         rgb_table=13, vert_colors=bytscl(alog10(datapoints[rr]))
endif



; Apply coordinate transforms and rotations
; 
; **For geometric mode, transformations will be
;   applied inside thm_part_slice2d_geo instead.**
;------------------------------------------

; Get mag data
bfield = thm_part_slice2d_getmag(ds, mag_data=mag_data, trange=trange, rotation=rotation, fail=fail)
if n_elements(bfield) eq 1 then return


; Get bulk velocity (in m/s)
vbulk = thm_part_slice2d_getvel(ds, vel_data=vel_data, trange=trange, rotation=rotation, fail=fail)
if n_elements(vbulk) eq 1 then return


; Get Sun vector
sunvec = thm_part_slice2d_getsun(ds, trange=trange, fail=fail) 


; Subtract bulk velocity vector?
if keyword_set(subtract) and type ne 0 then begin
  thm_part_slice2d_subtract,  velocities, vbulk, vel_data=vel_data
endif


; Transform data to GSM or GSE if requested
thm_part_slice2d_cotrans, velocities, bfield=bfield, vbulk=vbulk, sunvec=sunvec, $
                          coord=coord, probe=probe, type=type, twin=trange, $
                          matrix=ct, fail=fail
if keyword_set(fail) then return


; Transform data to special coordinates including field alligned (e.g. BV, PERP, Xgse, Rgeo)
thm_part_slice2d_rotate, velocities, bfield=bfield, vbulk=vbulk, sunvec=sunvec, $ 
                         type=type, rotation=rotation, probe=probe, $
                         mag_data=mag_data, $ ;need bfield as tplot var
                         twin=trange, matrix=rot, fail=fail
if keyword_set(fail) then return


; Transform data to slice plane's coordinates
thm_part_slice2d_orientslice, velocities, vbulk=vbulk, sunvec=sunvec, type=type, $
                              slice_x=slice_x, slice_z=slice_z, matrix=mt, $ 
                              fail=fail 
if keyword_set(fail) then return 
                                  


; Misc.
;------

; Convert velocities to km/s
factor = 1000.
velocities = temporary(velocities)/factor
vbulk = vbulk/factor
if keyword_set(dv) then dv = dv/factor
if keyword_set(vrange) then vrange = vrange/factor


;  ;Visualization for testing:
;  rr = where(datapoints ne 0)
;  rg = [-max(velocities),max(velocities)]
;  iplot, velocities[rr,0], velocities[rr,1], velocities[rr,2], linestyle=6, sym_index=1, $
;         xrange = rg, yrange=rg, zrange=rg, $
;         rgb_table=13, vert_colors=bytscl(alog10(datapoints[rr]))


; Sort transformed velocity grid
sorted = sort(velocities[*,0])
velocities = velocities[sorted,*]
if keyword_set(dp) then begin
  dp = dp[sorted]
  dt = dt[sorted]
  dv = dv[sorted]
endif
datapoints = datapoints[sorted]



; Create slice:
; 
; TYPE=0   Geometric Method
; TYPE=1   2D Nearest Neighbor Interpolation
; TYPE=2   2D Linear Interpolation
; TYPE=3   3D Linear Interpolation
;------------------------------------------


; Linear 2D interpolation from thm_esa_slice2D
if type eq 2 then begin

  dprint, dlevel=4, 'Using 2d linear interpolation'
  thm_part_slice2d_2di, datapoints, velocities, resolution, $
                          thetarange=thetarange, zdirrange=zdirrange, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          fail=fail

; Linear 3D Interpolation
endif else if type eq 3 then begin

  dprint, dlevel=4, 'Using 3d linear interpolation'
  thm_part_slice2d_3di, datapoints, velocities, resolution, $
                          orange=orange, displacement=displacement, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          fail=fail


; 2D Nearest Neighbor (for testing only)
endif else if type eq 1 then  begin

  dprint, dlevel=4, 'Using 2d nearest neighbor'
  thm_part_slice2d_nn, datapoints, velocities, resolution, slice_orient, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          slice_width=slice_width, $
                          shift=keyword_set(subtract) ? vbulk:0, $
                          fail=fail

; Geometric Method
endif else begin

  dprint, dlevel=4, 'Using geometric method'
  thm_part_slice2d_geo, datapoints, velocities, resolution, $
                        dp, dt, dv, displacement=displacement, $
                        average_angle=average_angle, $
                        ct=ct, rot=rot, mt=mt, $
                        msg_obj=msg_obj, msg_prefix=msg, $
                        part_slice=part_slice, $
                        xgrid=xgrid, ygrid=ygrid, $
                        fail=fail

endelse

if keyword_set(fail) then begin
  dprint, dlevel=1, fail
  return
endif


; Apply smoothing
if keyword_set(smooth) then begin
  thm_part_slice2d_smooth, part_slice, smooth
endif



; Pass out slice information for plotting
;----------------------------------------

;Misc
for i=0, n_elements(ds)-1 do begin
  ;dist type name
  dname = keyword_set(dname) ? [dname,((*ds[i]).data_name)[0]]:((*ds[i]).data_name)[0]
  
  ;time range of particle data
  times_ind = thm_part_slice2d_intrange(ds[i], trange, n=ndat)
  if ndat gt 0 then begin
    if keyword_set(tr) then begin
      tr = [ tr[0] < min((*ds[i])[times_ind].time), $
             tr[1] > max((*ds[i])[times_ind].end_time) ]
    endif else begin
      tr = [ min((*ds[i])[times_ind].time), $
             max((*ds[i])[times_ind].end_time) ]
    endelse
  endif

endfor
dname = strjoin(dname,'/')

;Velocity range
if keyword_set(displacement) then begin
  vrange = sqrt(  (vrange^2 - displacement^2) > 0. )
endif 


slice_struct = {  $
             ; Data
              data: temporary(part_slice), $  ;data values
              xgrid: xgrid, $    ;x-axis values
              ygrid: ygrid, $    ;y-axis values
             ; Metadata
              probe: probe, $    ;probe
              dist: dname, $     ;type of distribution (string)
              coord: coord, $    ;coordinates used (string)
              rot: rotation, $   ;rotation used (string)
              units: units, $    ;units (string)
              twin: timewin, $   ;length of slice in sec
              type: type, $      ;integer specifying slice type (0,2,3)
              range: orange, $   ;data range before interpolation
              trange: tr, $      ;total time range of used distributions
              ndists: n_elements((*ds[0])), $ ;# distributions used
              vrange: vrange , $              ;instrument velocity range
             ; Support Data
              shift: keyword_set(subtract) ? vbulk:0, $ ;for nn plotting
              bulk: keyword_set(vbulk) ? vbulk:0, $     ;bulk velocity vector
              sunvec: keyword_set(sunvec) ? sunvec:0, $  ;sun vector
              coord_m: keyword_set(ct) ? ct:-1, $ ;DSL to COORD matrix
              rot_m: rot, $                       ;COORD to ROTATION matrix
              orient_m: keyword_set(mt) ? mt:-1 $ ;COORD/ROTATION to slice plane coords matrix 
              }


msg = 'Finished slice at '+time_string(slice_time, format=5)
dprint, dlevel=2, msg 
if obj_valid(msg_obj) then msg_obj->update, msg 


; Fill dummy vars if present to preserve backwards compatability
if arg_present(xgrid) then xgrid = 0
if arg_present(ygrid) then ygrid = 0
if arg_present(slice_info) then slice_info = 0


return

end
