;+
; 
; Purpose:
;  Checks for changes compatability between distributions and returns boolean. 
; 
; Arguments/Keywods:
;   dist1/dist2: 3d data structures to be compared
;   msg: string describing discrepancy
;-
function thm_part_slice2d_checkbins, dist1, dist2, msg=msg

    compile_opt idl2, hidden

  diff = ''
  
  if in_set(dist1.theta eq dist2.theta,0) then diff += 'THETA '
  if in_set(dist1.phi eq dist2.phi,0) then diff += 'THETA '
  if in_set(dist1.energy eq dist2.energy,0) then diff += 'ENERGY '
  if dist1.mass ne dist2.mass then diff += 'MASS '
  if diff ne '' then begin
    diff = strjoin(strsplit(diff,/extract),', ')
    msg = diff+' values differ between distributions at '+ $
           time_string(dist1.time)+' and '+time_string(dist2.time)
    return, 0
  endif

  return, 1

end



;+
; 
; Purpose:
;  Converts one count value to requested units and returns converted data array.
; 
; Arguments:
;   dat: 3d data structure to be used
;   units: string describing new units
;
;-
function thm_part_slice2d_onecnt, dat, units

    compile_opt idl2, hidden


  ;ensure data is in counts
  ;( 2013-02-25 This conversion seems unnecessary as the "data" tag 
  ;  is the only element of the structure altered by this conversion ) 
  if strlowcase(dat.units_name) ne 'counts' then begin
    cdat = conv_units(dat,'counts')
    if strlowcase(dat.units_name) ne 'counts' then begin
      dprint, dlevel=0, 'WARNING: Error converting units, one-count level cannot be asertained' 
      return, 0
    endif
  endif else begin
    cdat = dat
  endelse
  
  cdat.data[*,*] = 1.

  cdat = conv_units(temporary(cdat),strlowcase(units))
  
  return, cdat.data
      
end



;+
; 
; Purpose:
;  Returns an array of gapless energy boundaries.  The number of 
;  elements returned will always be N+1 for N energy levels.
; 
; Notes:
;  Energy levels are ordered differently between ESA/SST
;
;-
function thm_part_slice2d_nrg, thedata

    compile_opt idl2, hidden

    n = thedata.nenergy

    energies = fltarr(size(thedata.energy,/dim)+[1,0])
    
    ; use midpoints
    energies[1:n-1,*] = (thedata.energy[0:n-2,*] + thedata.energy[1:n-1,*]) / 2.
    
    ; top/bottom energies
    energies[0,*] = thedata.energy[0,*] + (thedata.energy[0,*] - energies[1,*])
    energies[n,*] = thedata.energy[n-1,*] + (thedata.energy[n-1,*] - energies[n-1,*])

    return, energies
  
end


;+
; 
; Purpose:
;  Converts spherical coordinates to cartesian
; 
; Arguments:
;   r: N element array of radial values (can be any dimensions)
;   theta: N element array of theta values ( " )
;   phi: N element array of phi values ( " )
;   vec: Nx3 array of cartesian values in x,y,z order
;   
;-
pro thm_part_slice2d_s2c, r,theta,phi, vec

    compile_opt idl2, hidden

  rd = !dpi/180.
  n = n_elements(r)
  a = cos(rd*theta)
  vec = [ [reform(a * cos(rd*phi) * r, n)], $
          [reform(a * sin(rd*phi) * r, n)], $
          [reform(sin(rd*theta) * r, n)  ]   ]
end



;+
;
; Purpose:
;  Copy particle data into new array.
;  
;  Calculate the center of all bins in cartesian coordinates.
;  
;  Corresponding arrays for d-phi, d-theta, and d-v will be 
;  calculated if requested.
;  
; Arguments/Keywords:
;   thedata: 3D data structure
;   grid_set: (bool) Flag to track whether the cartesian velocity 
;                    components have been set yet. 
;   ncounts: New variable the particle data is copied to
;   velocities: Nx3 array of cartesian coordiantes for N bins
;     dp: N element array of bin width in phi
;     dt: N element array of bin width in theta
;     dv: N element array of bin width in velocity
;
;-
pro thm_part_slice2d_getBASIC, thedata, grid_set=grid_set, $
                           ncounts = ncounts, $
                           velocities = velocities, $
                           dp=dp, dt=dt, dv=dv, $
                           fail = fail

    compile_opt idl2, hidden

  thm_part_slice2d_const, q=q, conv=conv

  ; Calculate cartesian velocity componenets of each bin's center
  if ~keyword_set(grid_set) then begin

    n = thedata.nenergy

    dphi = 0.5 * thedata.dphi
    dtheta = 0.5 * thedata.dtheta
    
    ; Determine gapless energy boundaries.
    energies = thm_part_slice2d_nrg(thedata)

    ; mass of current species in kg
    mass = thedata.mass / conv ; (mass ~= proton | electron mass)

    ; Center of each bin (re-calculated) (centered)
    v = sqrt(2 * q * (energies[0:n-1,*]+energies[1:n,*])/ 2 / mass)
    thm_part_slice2d_s2c, v, thedata.theta, thedata.phi, velocities


    ; Phi, theta, and velocity with of each bin
    if arg_present(dp) then begin
      dp = reform(thedata.dphi, n_elements(thedata.dphi))
      dt = reform(thedata.dtheta, n_elements(thedata.dtheta)) 
      dv = abs(sqrt(2 * q * energies[1:n,*]/mass) - $
               sqrt(2 * q * energies[0:n-1,*]/mass)  )
    endif
    
    grid_set = 1b ;set indicator flag that cartesian velocity 
                  ;components have been calculated
    
  endif

  ; Array of counts for each nrg/angle bin...same size as xvels
  ncounts = thedata.data


  return

end



;+
;
; Purpose:
;  Copy particle data into new array and calculate a new set of interpolated 
;  data points in cartesian coordinates.
;  
;  Angular data from each energy bin will be spherically interpolated using the 
;  nearest neighbor. Energy bins are interpolated next, again with the nearest 
;  neighbor. Finally, the cartesian coordinates for the new interpolated points 
;  are returned. 
;  
; Arguments/Keywords:
;   thedata: 3D data structure
;   grid_set: (bool) Flag to track whether the cartesian velocity 
;                    components have been set yet. 
;   bins: Boolean array indicating which bins are OK to use
;         (invalid bins are zeroed so as to not be overwritten)
;   regrid: 3 Element array specifying the new number of points in 
;           phi, theta, and energy respectively.
;   ncounts: New variable the particle data is copied to
;   velocities: Nx3 array of cartesian coordiantes for N bins
;
;-
pro thm_part_slice2d_getREGRID, thedata, grid_set=grid_set, regrid=regrid, $
                           ncounts=ncounts, bins=bins, $
                           velocities=velocities, $
                           fail=fail

    compile_opt idl2, hidden


  thm_part_slice2d_const, q=q, conv=conv

  if n_elements(regrid) lt 3 then begin
    fail = 'REGRID keyword must contain 3 elements [phi,theta,energy]'
    return
  endif

  method = 'NearestNeighbor'
  phigrid = interpol( [0.,360], regrid[0])
  thetagrid = interpol( minmax(thedata.theta), regrid[1])
  
  enum = round(regrid[2]/thedata.nenergy) > 1
  seed_tmp = findgen(enum)/enum + 1./(2*enum)
  if ~keyword_set(grid_set) and enum gt 1 then dprint, dlevel=4, 'Interpolating each energy to '+strtrim(enum,2)+' energies.'

  ; mass of current species in kg
  mass = thedata.mass / conv ; (mass ~= proton | electron mass)

  ; zero invalid bins
  thedata.data = thedata.data * bins
  
  ; Loop over energy bins. Data will be spherically interpolated
  ; using the nearest neighbor at each energy. Those values will
  ; then be copied to cover the range of the energy bin. 
  ; ---------------------------------
  for i=0, thedata.nenergy-1 do begin

    ; Angles and data must be copied into new variable for triangulation/gridding
    phitemp = reform(thedata.phi[i,*])
    thetatemp = reform(thedata.theta[i,*])
    tempcounts = reform(thedata.data[i,*])
    
    if regrid[0] lt n_elements(phitemp[uniq(phitemp, sort(phitemp))]) or $
       regrid[1] lt n_elements(thetatemp[uniq(thetatemp, sort(thetatemp))]) then begin
       dprint, dlevel=1, "WARNING: Current regrid settings are below the current data's resolution."
    endif
    
    ; Create a (required) sphereical triangulation
    qhull, phitemp, thetatemp, triangles, sphere=dummy
    
    ; Interpolate this energy's data onto a regular spherical grid
    newgrid = griddata(phitemp, thetatemp, tempcounts, $
                       method=method, /sphere, /degrees, $
                       triangles=triangles, $
                       /grid, xout=phigrid, yout=thetagrid)

    ; Replicate data if interpolating across energies
    if keyword_set(enum) then begin
      newgrid = rebin(newgrid, [size(newgrid,/dim),enum], /sample)
    endif

    ; Concatenate data arrays (needs to have single dimension)
    tempgrid = ~keyword_set(tempgrid) ? reform(newgrid,n_elements(newgrid)): $
                             [tempgrid, reform(newgrid,n_elements(newgrid))]


    ; Only calculate cartesian velocity components once.
    ; Changes in angle bins are checked elsewhere.
    if ~keyword_set(grid_set) then begin
      
      ;energy value should not be changing across the selected bins (checked above)
      if keyword_set(enum) then begin
        denergyGrid = (seed_tmp * median(thedata.denergy[i,*])) + $
                                  median(thedata.energy[i,*])
        energyvalue = sqrt(2*q*(denergyGrid[0])/mass)
      endif else begin
        energyvalue = sqrt(2*q*median(thedata.energy[i,*])/mass)
      endelse

      ; x & y velocity arrays
      x = cos(!pi/180*phigrid) # cos(!pi/180*thetagrid) * energyvalue
      y = sin(!pi/180*phigrid) # cos(!pi/180*thetagrid) * energyvalue

      ; z velocity array corresponding to x & y arrays
      ztmp = sin(!pi/180*thetagrid) * energyvalue
      z = rebin(transpose(ztmp), size(x,/dim), /sample)

      ; concatenate to arrays from previous loops
      first = ~keyword_set(xvels)
      xvels = first ? reform(x,n_elements(x)):[xvels, reform(x,n_elements(x))]
      yvels = first ? reform(y,n_elements(y)):[yvels, reform(y,n_elements(y))]
      zvels = first ? reform(z,n_elements(z)):[zvels, reform(z,n_elements(z))]

      ; add velocities corresponding interpolated energies
      if keyword_set(enum) then begin
        factorGrid = sqrt(denergyGrid / denergyGrid[0])
        for l=1, enum-1 do begin ; 0th was already done above
          xvels = [temporary(xvels), reform(factorGrid[l] * x,n_elements(x))]
          yvels = [temporary(yvels), reform(factorGrid[l] * y,n_elements(y))]
          zvels = [temporary(zvels), reform(factorGrid[l] * z,n_elements(z))]
        endfor
      endif
      
      
    endif
    
  endfor

  if ~keyword_set(grid_set) then begin
    velocities = [ [xvels], [yvels], [zvels] ]
    grid_set = 1b  ;set indicator flag that cartesian velocity 
                   ;components have been calculated
  endif

  ; Pass out interpolated data
  ncounts = temporary(tempgrid)

  ; All bins are valid
;  bins = replicate(1b, size(ncounts,/dim))
  bins = ncounts gt 0

  return

end



;+
;
; Name: thm_part_slice2d_getXYZ
; 
; Purpose: Helper function for thm_part_slice2d.pro
; 
;          Converts data into the requested units and calculates cartesian 
;          velocity bins based off the distributions energy and angle bins.
;          
;          If regridding, then all data will first be interpolated onto a regular 
;          grid in spherical coordinates.
;          
;          If geometric method will be used then phi, theta, and enegery ranges 
;          will also be returned via keywords.
; 
; Arguments: 
;     datStructs: Array of 3d data structures
;     units: String descriping the units desired
;     type: Number specifying the type of slice that will be created
;           (see thm_part_slice2d)
;     
; 
; Input Keywords:
;   count_threshold: Mask out bins with counts below the specified value.
;                  If not already working in counts the value will
;                  be converted to the specified units.
;                 (after averaging -> before averaging - 2012-12-21)
;                 (allow specified value - 2013-02-25)
;   regrid: 3 Element array specifying the new number of points desired in 
;           phi, theta, and energy respectively.
;   erange: Two element array specifying min/max energies to be used
;    
;    
; Output Keywords:
;   velocities: N x 3 array of velocity components (v_x, v_y, v_z) in km/s
;               corresponding to the center of each bin (except when regridding).
;   datapoints: N element array of datapoints
;   
;   orange: Two element array; min/max of the data in specified units (with constraints applied)
;   vrange; Two element array; min/max velocities from the dist. (with constraints applied)
;          
;   (for geometric only)
;     dp: N element array of bin width in phi
;     dt: N element array of bin width in theta
;     dv: N element array of bin width in velocity         
; 
;-

pro thm_part_slice2d_getXYZ, datStructs, units, type, $
                             count_threshold=count_threshold, regrid=regrid, erange=erange, $
                             vrange=vrange, orange=orange, $
                             velocities=velocities, $
                             dp=dp, dt=dt, dv=dv, $
                             datapoints=datapoints, $
                             fail=fail, $
                             _extra=_extra

    compile_opt idl2, hidden


  thm_part_slice2d_const, q=q, conv=conv


  ;Loop over distributions
  ;-----------------------
  for in=0, n_elements(datStructs)-1 do begin
    
    thedata=datStructs[in]

    ; Verify that bins do not differ from the last distribution's
    ; In general, this code assumes it is being called on a similar set of 3d data structs
    if ~thm_part_slice2d_checkbins(thedata, datstructs[in-1 > 0], msg=msg) then begin 
      fail = msg
      dprint, fail
      return
    endif
    
    ;Zero bins below count threshold if requested
    if keyword_set(count_threshold) then begin
      ltidx = where(thedata.data lt count_threshold and thedata.data ne 0., nlt)
      if nlt gt 0 then begin
        thedata.data[ltidx] = 0.
        dprint, dlevel=4, 'Removing '+strtrim(nlt,2)+' bins below '+strtrim(count_threshold)+$
                          ' count(s) from "'+thedata.data_name+'"'
      endif
    endif

    ; Convert to specified units ('df' is default)
    thedata = conv_units(temporary(thedata),strlowcase(units),_extra=_extra)

    ;Verify valid data
    if thedata.valid ne 1 then begin
      dprint, dlevel=4,  'No valid data for disribution at: '+time_string(thedata.time)
      continue
    endif
  
    ; Find active, valid bins.
    bins = (thedata.bins ne 0) and (finite(thedata.data) eq 1)

    ; Find bins within energy limits
    if keyword_set(erange) then begin
      n = thedata.nenergy
      energies = thm_part_slice2d_nrg(thedata)
      ecenters = (energies[0:n-1,*]+energies[1:n,*])/ 2
      bins = bins and (ecenters ge erange[0] and ecenters le erange[1])
    endif

    ; Get cartesian coordinates
    if type eq 0 then begin    
    
      ; Get center of each bin plus d-phi, d-theta, d-velocity
      thm_part_slice2d_getBASIC, thedata, grid_set=grid_set, fail=fail, $
                               ncounts = ncounts, $
                               velocities = velocities, $
                               dp=dp, dt=dt, dv=dv
                               
    endif else if keyword_set(regrid) then begin
    
      ; Regrid in spherical coordinates then determine x,y,z velocity components.
      thm_part_slice2d_getREGRID, thedata, grid_set=grid_set, fail=fail, $
                               regrid=regrid, bins=bins, $
                               ncounts = ncounts, $
                               velocities = velocities
    
    endif else begin
    
      ; Get center of each bin
      thm_part_slice2d_getBASIC, thedata, grid_set=grid_set, fail=fail, $
                               ncounts = ncounts, $
                               velocities = velocities
    
    endelse
                               
    if keyword_set(fail) then return
  
  
    ; Sum of counts at each time sample
    ncounts_t = keyword_set(ncounts_t) ? ncounts_t+ncounts:ncounts


    ; Keep track of valid bins within energy range.  This array will later be used
    ; to average bins and discard any that are out of range or invalid.
    weight_t = keyword_set(weight_t) ? (weight_t + bins):bins
    
  endfor
  ;--------------------------
  ;End loop over distributions
  
  
  ; Average data over number of time samples containging valid measurements
  datapoints = ncounts_t / (weight_t > 1)
  
  
;**One-count removal now done before averaging** 2012-12-21 
;  ;Zero bins whose average is below the one count level
;  if keyword_set(onecount) then begin
;    onecnt = thm_part_slice2d_onecnt(datstructs[0], units)
;    if keyword_set(regrid) then begin
;       thedata.data = onecnt
;       thm_part_slice2d_getREGRID, thedata, 2, regrid=regrid, ncounts=onecnt, fail=fail
;    endif
;    if n_elements(onecnt) eq n_elements(ncounts_t) then begin
;      ltidx = where(datapoints lt onecnt, nlt)
;      if nlt gt 0 then datapoints[ltidx] = 0.
;      dprint, dlevel=4, 'Removing '+strtrim(nlt,2)+' bins below one count from "'+thedata.data_name+'"'
;    endif else dprint, 'Error matching one-count data to distribution. ' + $
;                       'Bins below one count could not be subtracted!'
;  endif
  
  
  ; Remove non-valid bins
  valid = where(weight_t gt 0, nvalid)
  if nvalid gt 0 then begin
    datapoints = datapoints[valid]
    velocities = velocities[valid,*]
    if keyword_set(dp) then begin
      dp = dp[valid]
      dt = dt[valid]
      dv = dv[valid]
    endif
  endif else begin
    fail = 'No valid data in distribution(s).  '+ $
           'This is may be due to stringent energy limits or '+ $
           'A lack of data above the one-count level.'
    dprint, dlevel=1, fail
    return
  endelse


  ; Pass out data range and velocity range
  orange = minmax(datapoints,/pos)
  
  ;adjust for energy restrictions
  if keyword_set(erange) then begin
    er = [erange[0], max(energies) < erange[1]]
    
    ;snap to bin boundaries
    energies = energies[0:(size(energies,/dim))[0]-1]
    ecenters = ecenters[0:(size(ecenters,/dim))[0]-1]
    uidx = where(ecenters le er[1], nu)
    if nu gt 0 then er[1] = max(energies[[uidx,uidx+1]])
    lidx = where(ecenters ge er[0], nl)
    if nl gt 0 then er[0] = min(energies[[lidx,lidx+1]])

  endif else begin
    energies = thm_part_slice2d_nrg(thedata)
    er = minmax(energies)
  endelse

  vrange = sqrt(2 * q * er/(thedata.mass/conv))


  return
  
end
